/** {box-sizing: border-box;}*/

/*.img-magnifier-container {*/
/*  position: relative;*/
/*}*/

/*.img-magnifier-glass {*/
/*  position: absolute;*/
/*  border: 3px solid #000;*/
/*  !*border-radius: 50%;*!*/
/*  cursor: none;*/
/*  !*Set the size of the magnifier glass:*!*/
/*  width: 400px;*/
/*  height: 100px;*/
/*}*/

#content{
  background-color:#F1F1F1;
  height: auto;
}
/*.container-fluid{*/
/*  background-color:white;*/
/*}*/

.card-header #white-header{
  background-color: white;
}

* {box-sizing: border-box;}

.img-zoom-container {
}

#myimage{
  /*border: 4px solid white;*/
}

.img-zoom-lens {
  position: absolute;
  border: 1px solid #d4d4d4;
  /*set the size of the lens:*/
  width: 100%;
  height: 10%;
}

.img-zoom-result {
  position: fixed;

  z-index: 40;
  visibility: hidden;
  /*set the size of the result div:*/
  width: 1000px;
  height: 200px;
  border: 4px solid white;

}

